# -------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
# -------------------------------------------------------------

# Autogenerated By   : src/main/python/generator/generator.py
# Autogenerated From : scripts/builtin/ppca.dml

from typing import Dict, Iterable

from systemds.operator import OperationNode, Matrix, Frame, List, MultiReturn, Scalar
from systemds.utils.consts import VALID_INPUT_TYPES


def ppca(X: Matrix,
         **kwargs: Dict[str, VALID_INPUT_TYPES]):
    """
     This script performs Probabilistic Principal Component Analysis (PCA) on the given input data.
     It is based on paper: sPCA: Scalable Principal Component Analysis for Big Data on Distributed
     Platforms. Tarek Elgamal et.al.
    
    
    
    :param X: n x m input feature matrix
    :param k: indicates dimension of the new vector space constructed from eigen vectors
    :param maxi: maximum number of iterations until convergence
    :param tolobj: objective function tolerance value to stop ppca algorithm
    :param tolrecerr: reconstruction error tolerance value to stop the algorithm
    :param verbose: verbose debug output
    :return: Output feature matrix with K columns
    :return: Output dominant eigen vectors (can be used for projections)
    """

    params_dict = {'X': X}
    params_dict.update(kwargs)
    
    vX_0 = Matrix(X.sds_context, '')
    vX_1 = Matrix(X.sds_context, '')
    output_nodes = [vX_0, vX_1, ]

    op = MultiReturn(X.sds_context, 'ppca', output_nodes, named_input_nodes=params_dict)

    vX_0._unnamed_input_nodes = [op]
    vX_1._unnamed_input_nodes = [op]

    return op
