# -------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
# -------------------------------------------------------------

# Autogenerated By   : src/main/python/generator/generator.py
# Autogenerated From : scripts/builtin/smape.dml

from typing import Dict, Iterable

from systemds.operator import OperationNode, Matrix, Frame, List, MultiReturn, Scalar
from systemds.utils.consts import VALID_INPUT_TYPES


def smape(X: Matrix,
          Y: Matrix,
          **kwargs: Dict[str, VALID_INPUT_TYPES]):
    """
     Returns the symmetric means absolute percentage error between the two inputs
    
     Monash Time Series Forecasting Archive
     Rakshitha Godahewaa, Christoph Bergmeira, Geoffrey I. Webba, Rob J. Hyndmanb,
     Pablo Montero-Mansoc
    
     Another Look at Measures of Forecast Accuracy, R. J. Hyndman and A. B. Koehler, 2006. 
    
    
    
    :param X: First Matrix to compare
    :param Y: Second Matrix to compare
    :param P: Quantiles to extract as well if empty matrix not calculated
    :return: The symmetric mean absolute percentage error
    :return: Quantiles calculated
    """

    params_dict = {'X': X, 'Y': Y}
    params_dict.update(kwargs)
    
    vX_0 = Matrix(X.sds_context, '')
    vX_1 = Matrix(X.sds_context, '')
    output_nodes = [vX_0, vX_1, ]

    op = MultiReturn(X.sds_context, 'smape', output_nodes, named_input_nodes=params_dict)

    vX_0._unnamed_input_nodes = [op]
    vX_1._unnamed_input_nodes = [op]

    return op
